@@ -83,10 +83,9 @@ exchange_folder_append_message_data (CamelFolder *folder,
 		stream_cache = camel_data_cache_add (
 			exch->cache, "cache", new_uid, NULL);
 		if (stream_cache) {
-			camel_stream_write (stream_cache,
-					    (gchar *) message->data,
-					    message->len, NULL);
-			camel_stream_flush (stream_cache, NULL);
+			if (camel_stream_write (stream_cache, (gchar *) message->data, message->len, NULL) <= 0 ||
+			    camel_stream_flush (stream_cache, NULL) == -1)
+				camel_data_cache_remove (exch->cache, "cache", new_uid, NULL);
 			g_object_unref (stream_cache);
 		}
 		if (appended_uid)
@@ -118,6 +117,17 @@ exchange_folder_get_message_data (CamelFolder *folder,
 	store = CAMEL_EXCHANGE_STORE (parent_store);
 
 	stream = camel_data_cache_get (exch->cache, "cache", uid, NULL);
+	if (stream) {
+		CamelStream *null_stream = camel_stream_null_new ();
+
+		camel_stream_reset (stream, NULL);
+		if (camel_stream_write_to_stream (stream, null_stream, NULL) <= 0) {
+			stream = NULL;
+		}
+
+		g_object_unref (null_stream);
+	}
+
 	if (stream) {
 		ba = g_byte_array_new ();
 		stream_mem = camel_stream_mem_new ();
@@ -148,8 +158,9 @@ exchange_folder_get_message_data (CamelFolder *folder,
 		return NULL;
 	}
 
-	camel_stream_write (stream, (gchar *) ba->data, ba->len, NULL);
-	camel_stream_flush (stream, NULL);
+	if (camel_stream_write (stream, (gchar *) ba->data, ba->len, NULL) <= 0 ||
+	    camel_stream_flush (stream, NULL) == -1)
+		camel_data_cache_remove (exch->cache, "cache", uid, NULL);
 	g_object_unref (stream);
 
 	return ba;
@@ -496,6 +507,8 @@ exchange_folder_get_message (CamelFolder *folder,
 	if (!ba)
 		return NULL;
 
+	g_byte_array_append (ba, (guint8 *) "", 1);
+
 	while ((ba->len > 10 && g_str_has_prefix ((const gchar *)ba->data, "MAIL FROM:")) ||
 	       (ba->len >  8 && g_str_has_prefix ((const gchar *)ba->data, "RCPT TO:")) ||
 	       (ba->len >  2 && (ba->data[0] == '\n' || ba->data[1] == '\n'))) {
@@ -511,6 +524,9 @@ exchange_folder_get_message (CamelFolder *folder,
 			break;
 	}
 
+	if (ba->len > 0 && ba->data[ba->len - 1] == 0)
+		g_byte_array_remove_index (ba, ba->len - 1);
+
 	stream = camel_stream_mem_new_with_byte_array (ba);
 
 	crlffilter = camel_mime_filter_crlf_new (CAMEL_MIME_FILTER_CRLF_DECODE, CAMEL_MIME_FILTER_CRLF_MODE_CRLF_ONLY);
